# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["TaskStartStreamParams"]


class TaskStartStreamParams(TypedDict, total=False):
    agent_type: str

    environment_id: str

    instruction: str

    kind: Literal["desktop", "browser"]

    max_steps: int

    model: str

    persistent: bool

    screenshot_mode: Literal["url", "base64"]

    temperature: float

    viewport_height: int

    viewport_width: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .chat import (
    ChatResource,
    AsyncChatResource,
    ChatResourceWithRawResponse,
    AsyncChatResourceWithRawResponse,
    ChatResourceWithStreamingResponse,
    AsyncChatResourceWithStreamingResponse,
)
from .agent import (
    AgentResource,
    AsyncAgentResource,
    AgentResourceWithRawResponse,
    AsyncAgentResourceWithRawResponse,
    AgentResourceWithStreamingResponse,
    AsyncAgentResourceWithStreamingResponse,
)
from .models import (
    ModelsResource,
    AsyncModelsResource,
    ModelsResourceWithRawResponse,
    AsyncModelsResourceWithRawResponse,
    ModelsResourceWithStreamingResponse,
    AsyncModelsResourceWithStreamingResponse,
)
from .computers import (
    ComputersResource,
    AsyncComputersResource,
    ComputersResourceWithRawResponse,
    AsyncComputersResourceWithRawResponse,
    ComputersResourceWithStreamingResponse,
    AsyncComputersResourceWithStreamingResponse,
)
from .responses import (
    ResponsesResource,
    AsyncResponsesResource,
    ResponsesResourceWithRawResponse,
    AsyncResponsesResourceWithRawResponse,
    ResponsesResourceWithStreamingResponse,
    AsyncResponsesResourceWithStreamingResponse,
)

__all__ = [
    "AgentResource",
    "AsyncAgentResource",
    "AgentResourceWithRawResponse",
    "AsyncAgentResourceWithRawResponse",
    "AgentResourceWithStreamingResponse",
    "AsyncAgentResourceWithStreamingResponse",
    "ComputersResource",
    "AsyncComputersResource",
    "ComputersResourceWithRawResponse",
    "AsyncComputersResourceWithRawResponse",
    "ComputersResourceWithStreamingResponse",
    "AsyncComputersResourceWithStreamingResponse",
    "ResponsesResource",
    "AsyncResponsesResource",
    "ResponsesResourceWithRawResponse",
    "AsyncResponsesResourceWithRawResponse",
    "ResponsesResourceWithStreamingResponse",
    "AsyncResponsesResourceWithStreamingResponse",
    "ChatResource",
    "AsyncChatResource",
    "ChatResourceWithRawResponse",
    "AsyncChatResourceWithRawResponse",
    "ChatResourceWithStreamingResponse",
    "AsyncChatResourceWithStreamingResponse",
    "ModelsResource",
    "AsyncModelsResource",
    "ModelsResourceWithRawResponse",
    "AsyncModelsResourceWithRawResponse",
    "ModelsResourceWithStreamingResponse",
    "AsyncModelsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ComputerCreateParams", "Display"]


class ComputerCreateParams(TypedDict, total=False):
    auto_kill: bool
    """Deprecated: use idle_timeout_enabled"""

    context_id: str

    display: Display

    environment_id: str

    idle_timeout_enabled: bool
    """If true (default), kill session after inactivity"""

    inactivity_timeout_seconds: int
    """Idle timeout before kill"""

    kind: str
    """\"browser" (default) or "desktop" """

    max_lifetime_seconds: int
    """Max session duration in seconds"""

    persistent: bool
    """Persist cookies/storage state to DB on session teardown only if true"""

    stealth: object

    timeout_seconds: int
    """Deprecated: use max_lifetime_seconds"""

    use_advanced_proxy: bool
    """If true (browser sessions), use ADVANCED_PROXY_URL on session start"""


class Display(TypedDict, total=False):
    height: int

    scale: float

    width: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._streaming import Stream, AsyncStream
from ...types.agent import task_start_params, task_start_stream_params, task_inject_message_params
from ..._base_client import make_request_options
from ...types.agent.task_pause_response import TaskPauseResponse
from ...types.agent.task_start_response import TaskStartResponse
from ...types.agent.task_resume_response import TaskResumeResponse
from ...types.agent.task_start_stream_response import TaskStartStreamResponse
from ...types.agent.task_inject_message_response import TaskInjectMessageResponse
from ...types.agent.task_retrieve_status_response import TaskRetrieveStatusResponse

__all__ = ["TasksResource", "AsyncTasksResource"]


class TasksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TasksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/tzafon/lightcone-python#accessing-raw-response-data-eg-headers
        """
        return TasksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TasksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/tzafon/lightcone-python#with_streaming_response
        """
        return TasksResourceWithStreamingResponse(self)

    def inject_message(
        self,
        id: str,
        *,
        message: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskInjectMessageResponse:
        """
        Injects a message into a running agent task.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            path_template("/agent/tasks/{id}/messages", id=id),
            body=maybe_transform({"message": message}, task_inject_message_params.TaskInjectMessageParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskInjectMessageResponse,
        )

    def pause(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskPauseResponse:
        """
        Pauses a running agent task.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            path_template("/agent/tasks/{id}/pause", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskPauseResponse,
        )

    def resume(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskResumeResponse:
        """
        Resumes a paused agent task.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            path_template("/agent/tasks/{id}/resume", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskResumeResponse,
        )

    def retrieve_status(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskRetrieveStatusResponse:
        """
        Returns the current status/result for a task.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template("/agent/tasks/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskRetrieveStatusResponse,
        )

    def start(
        self,
        *,
        agent_type: str | Omit = omit,
        environment_id: str | Omit = omit,
        instruction: str | Omit = omit,
        kind: Literal["desktop", "browser"] | Omit = omit,
        max_steps: int | Omit = omit,
        model: str | Omit = omit,
        persistent: bool | Omit = omit,
        screenshot_mode: Literal["url", "base64"] | Omit = omit,
        temperature: float | Omit = omit,
        viewport_height: int | Omit = omit,
        viewport_width: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskStartResponse:
        """
        Starts an agent task and returns a task_id immediately.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/agent/tasks",
            body=maybe_transform(
                {
                    "agent_type": agent_type,
                    "environment_id": environment_id,
                    "instruction": instruction,
                    "kind": kind,
                    "max_steps": max_steps,
                    "model": model,
                    "persistent": persistent,
                    "screenshot_mode": screenshot_mode,
                    "temperature": temperature,
                    "viewport_height": viewport_height,
                    "viewport_width": viewport_width,
                },
                task_start_params.TaskStartParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskStartResponse,
        )

    def start_stream(
        self,
        *,
        agent_type: str | Omit = omit,
        environment_id: str | Omit = omit,
        instruction: str | Omit = omit,
        kind: Literal["desktop", "browser"] | Omit = omit,
        max_steps: int | Omit = omit,
        model: str | Omit = omit,
        persistent: bool | Omit = omit,
        screenshot_mode: Literal["url", "base64"] | Omit = omit,
        temperature: float | Omit = omit,
        viewport_height: int | Omit = omit,
        viewport_width: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Stream[TaskStartStreamResponse]:
        """
        Starts an agent task and streams events via SSE (Content-Type:
        text/event-stream).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "text/event-stream", **(extra_headers or {})}
        return self._post(
            "/agent/tasks/stream",
            body=maybe_transform(
                {
                    "agent_type": agent_type,
                    "environment_id": environment_id,
                    "instruction": instruction,
                    "kind": kind,
                    "max_steps": max_steps,
                    "model": model,
                    "persistent": persistent,
                    "screenshot_mode": screenshot_mode,
                    "temperature": temperature,
                    "viewport_height": viewport_height,
                    "viewport_width": viewport_width,
                },
                task_start_stream_params.TaskStartStreamParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
            stream=True,
            stream_cls=Stream[TaskStartStreamResponse],
        )


class AsyncTasksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTasksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/tzafon/lightcone-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTasksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTasksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/tzafon/lightcone-python#with_streaming_response
        """
        return AsyncTasksResourceWithStreamingResponse(self)

    async def inject_message(
        self,
        id: str,
        *,
        message: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskInjectMessageResponse:
        """
        Injects a message into a running agent task.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            path_template("/agent/tasks/{id}/messages", id=id),
            body=await async_maybe_transform({"message": message}, task_inject_message_params.TaskInjectMessageParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskInjectMessageResponse,
        )

    async def pause(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskPauseResponse:
        """
        Pauses a running agent task.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            path_template("/agent/tasks/{id}/pause", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskPauseResponse,
        )

    async def resume(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskResumeResponse:
        """
        Resumes a paused agent task.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            path_template("/agent/tasks/{id}/resume", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskResumeResponse,
        )

    async def retrieve_status(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskRetrieveStatusResponse:
        """
        Returns the current status/result for a task.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template("/agent/tasks/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskRetrieveStatusResponse,
        )

    async def start(
        self,
        *,
        agent_type: str | Omit = omit,
        environment_id: str | Omit = omit,
        instruction: str | Omit = omit,
        kind: Literal["desktop", "browser"] | Omit = omit,
        max_steps: int | Omit = omit,
        model: str | Omit = omit,
        persistent: bool | Omit = omit,
        screenshot_mode: Literal["url", "base64"] | Omit = omit,
        temperature: float | Omit = omit,
        viewport_height: int | Omit = omit,
        viewport_width: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TaskStartResponse:
        """
        Starts an agent task and returns a task_id immediately.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/agent/tasks",
            body=await async_maybe_transform(
                {
                    "agent_type": agent_type,
                    "environment_id": environment_id,
                    "instruction": instruction,
                    "kind": kind,
                    "max_steps": max_steps,
                    "model": model,
                    "persistent": persistent,
                    "screenshot_mode": screenshot_mode,
                    "temperature": temperature,
                    "viewport_height": viewport_height,
                    "viewport_width": viewport_width,
                },
                task_start_params.TaskStartParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TaskStartResponse,
        )

    async def start_stream(
        self,
        *,
        agent_type: str | Omit = omit,
        environment_id: str | Omit = omit,
        instruction: str | Omit = omit,
        kind: Literal["desktop", "browser"] | Omit = omit,
        max_steps: int | Omit = omit,
        model: str | Omit = omit,
        persistent: bool | Omit = omit,
        screenshot_mode: Literal["url", "base64"] | Omit = omit,
        temperature: float | Omit = omit,
        viewport_height: int | Omit = omit,
        viewport_width: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncStream[TaskStartStreamResponse]:
        """
        Starts an agent task and streams events via SSE (Content-Type:
        text/event-stream).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "text/event-stream", **(extra_headers or {})}
        return await self._post(
            "/agent/tasks/stream",
            body=await async_maybe_transform(
                {
                    "agent_type": agent_type,
                    "environment_id": environment_id,
                    "instruction": instruction,
                    "kind": kind,
                    "max_steps": max_steps,
                    "model": model,
                    "persistent": persistent,
                    "screenshot_mode": screenshot_mode,
                    "temperature": temperature,
                    "viewport_height": viewport_height,
                    "viewport_width": viewport_width,
                },
                task_start_stream_params.TaskStartStreamParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
            stream=True,
            stream_cls=AsyncStream[TaskStartStreamResponse],
        )


class TasksResourceWithRawResponse:
    def __init__(self, tasks: TasksResource) -> None:
        self._tasks = tasks

        self.inject_message = to_raw_response_wrapper(
            tasks.inject_message,
        )
        self.pause = to_raw_response_wrapper(
            tasks.pause,
        )
        self.resume = to_raw_response_wrapper(
            tasks.resume,
        )
        self.retrieve_status = to_raw_response_wrapper(
            tasks.retrieve_status,
        )
        self.start = to_raw_response_wrapper(
            tasks.start,
        )
        self.start_stream = to_raw_response_wrapper(
            tasks.start_stream,
        )


class AsyncTasksResourceWithRawResponse:
    def __init__(self, tasks: AsyncTasksResource) -> None:
        self._tasks = tasks

        self.inject_message = async_to_raw_response_wrapper(
            tasks.inject_message,
        )
        self.pause = async_to_raw_response_wrapper(
            tasks.pause,
        )
        self.resume = async_to_raw_response_wrapper(
            tasks.resume,
        )
        self.retrieve_status = async_to_raw_response_wrapper(
            tasks.retrieve_status,
        )
        self.start = async_to_raw_response_wrapper(
            tasks.start,
        )
        self.start_stream = async_to_raw_response_wrapper(
            tasks.start_stream,
        )


class TasksResourceWithStreamingResponse:
    def __init__(self, tasks: TasksResource) -> None:
        self._tasks = tasks

        self.inject_message = to_streamed_response_wrapper(
            tasks.inject_message,
        )
        self.pause = to_streamed_response_wrapper(
            tasks.pause,
        )
        self.resume = to_streamed_response_wrapper(
            tasks.resume,
        )
        self.retrieve_status = to_streamed_response_wrapper(
            tasks.retrieve_status,
        )
        self.start = to_streamed_response_wrapper(
            tasks.start,
        )
        self.start_stream = to_streamed_response_wrapper(
            tasks.start_stream,
        )


class AsyncTasksResourceWithStreamingResponse:
    def __init__(self, tasks: AsyncTasksResource) -> None:
        self._tasks = tasks

        self.inject_message = async_to_streamed_response_wrapper(
            tasks.inject_message,
        )
        self.pause = async_to_streamed_response_wrapper(
            tasks.pause,
        )
        self.resume = async_to_streamed_response_wrapper(
            tasks.resume,
        )
        self.retrieve_status = async_to_streamed_response_wrapper(
            tasks.retrieve_status,
        )
        self.start = async_to_streamed_response_wrapper(
            tasks.start,
        )
        self.start_stream = async_to_streamed_response_wrapper(
            tasks.start_stream,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tzafon import Lightcone, AsyncLightcone
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestChat:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_completion(self, client: Lightcone) -> None:
        chat = client.chat.create_completion(
            messages=[
                {
                    "content": "string",
                    "role": "developer",
                }
            ],
        )
        assert_matches_type(object, chat, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_completion_with_all_params(self, client: Lightcone) -> None:
        chat = client.chat.create_completion(
            messages=[
                {
                    "content": "string",
                    "role": "developer",
                    "name": "name",
                }
            ],
            add_generation_prompt=True,
            add_special_tokens=True,
            allowed_token_ids=[0],
            bad_words=["string"],
            cache_salt="cache_salt",
            chat_template="chat_template",
            chat_template_kwargs={"foo": "bar"},
            continue_final_message=True,
            documents=[{"foo": "string"}],
            echo=True,
            frequency_penalty=0,
            ignore_eos=True,
            include_reasoning=True,
            include_stop_str_in_output=True,
            kv_transfer_params={"foo": "bar"},
            length_penalty=0,
            logit_bias={"foo": 0},
            logits_processors=["string"],
            logprobs=True,
            max_completion_tokens=0,
            max_tokens=0,
            min_p=0,
            min_tokens=0,
            mm_processor_kwargs={"foo": "bar"},
            model="model",
            n=0,
            parallel_tool_calls=True,
            presence_penalty=0,
            priority=0,
            prompt_logprobs=0,
            reasoning_effort="low",
            repetition_penalty=0,
            request_id="request_id",
            response_format={
                "type": "text",
                "json_schema": {
                    "name": "name",
                    "description": "description",
                    "schema": {"foo": "bar"},
                    "strict": True,
                },
            },
            return_token_ids=True,
            return_tokens_as_token_ids=True,
            seed=-9007199254740991,
            skip_special_tokens=True,
            spaces_between_special_tokens=True,
            stop="string",
            stop_token_ids=[0],
            stream=True,
            stream_options={
                "continuous_usage_stats": True,
                "include_usage": True,
            },
            structured_outputs={
                "_backend": "_backend",
                "_backend_was_auto": True,
                "choice": ["string"],
                "disable_additional_properties": True,
                "disable_any_whitespace": True,
                "disable_fallback": True,
                "grammar": "grammar",
                "json": "string",
                "json_object": True,
                "regex": "regex",
                "structural_tag": "structural_tag",
                "whitespace_pattern": "whitespace_pattern",
            },
            temperature=0,
            tool_choice="none",
            tools=[
                {
                    "function": {
                        "name": "name",
                        "description": "description",
                        "parameters": {"foo": "bar"},
                    },
                    "type": "function",
                }
            ],
            top_k=0,
            top_logprobs=0,
            top_p=0,
            truncate_prompt_tokens=-1,
            use_beam_search=True,
            user="user",
            vllm_xargs={"foo": "string"},
        )
        assert_matches_type(object, chat, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create_completion(self, client: Lightcone) -> None:
        response = client.chat.with_raw_response.create_completion(
            messages=[
                {
                    "content": "string",
                    "role": "developer",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = response.parse()
        assert_matches_type(object, chat, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create_completion(self, client: Lightcone) -> None:
        with client.chat.with_streaming_response.create_completion(
            messages=[
                {
                    "content": "string",
                    "role": "developer",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = response.parse()
            assert_matches_type(object, chat, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncChat:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_completion(self, async_client: AsyncLightcone) -> None:
        chat = await async_client.chat.create_completion(
            messages=[
                {
                    "content": "string",
                    "role": "developer",
                }
            ],
        )
        assert_matches_type(object, chat, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_completion_with_all_params(self, async_client: AsyncLightcone) -> None:
        chat = await async_client.chat.create_completion(
            messages=[
                {
                    "content": "string",
                    "role": "developer",
                    "name": "name",
                }
            ],
            add_generation_prompt=True,
            add_special_tokens=True,
            allowed_token_ids=[0],
            bad_words=["string"],
            cache_salt="cache_salt",
            chat_template="chat_template",
            chat_template_kwargs={"foo": "bar"},
            continue_final_message=True,
            documents=[{"foo": "string"}],
            echo=True,
            frequency_penalty=0,
            ignore_eos=True,
            include_reasoning=True,
            include_stop_str_in_output=True,
            kv_transfer_params={"foo": "bar"},
            length_penalty=0,
            logit_bias={"foo": 0},
            logits_processors=["string"],
            logprobs=True,
            max_completion_tokens=0,
            max_tokens=0,
            min_p=0,
            min_tokens=0,
            mm_processor_kwargs={"foo": "bar"},
            model="model",
            n=0,
            parallel_tool_calls=True,
            presence_penalty=0,
            priority=0,
            prompt_logprobs=0,
            reasoning_effort="low",
            repetition_penalty=0,
            request_id="request_id",
            response_format={
                "type": "text",
                "json_schema": {
                    "name": "name",
                    "description": "description",
                    "schema": {"foo": "bar"},
                    "strict": True,
                },
            },
            return_token_ids=True,
            return_tokens_as_token_ids=True,
            seed=-9007199254740991,
            skip_special_tokens=True,
            spaces_between_special_tokens=True,
            stop="string",
            stop_token_ids=[0],
            stream=True,
            stream_options={
                "continuous_usage_stats": True,
                "include_usage": True,
            },
            structured_outputs={
                "_backend": "_backend",
                "_backend_was_auto": True,
                "choice": ["string"],
                "disable_additional_properties": True,
                "disable_any_whitespace": True,
                "disable_fallback": True,
                "grammar": "grammar",
                "json": "string",
                "json_object": True,
                "regex": "regex",
                "structural_tag": "structural_tag",
                "whitespace_pattern": "whitespace_pattern",
            },
            temperature=0,
            tool_choice="none",
            tools=[
                {
                    "function": {
                        "name": "name",
                        "description": "description",
                        "parameters": {"foo": "bar"},
                    },
                    "type": "function",
                }
            ],
            top_k=0,
            top_logprobs=0,
            top_p=0,
            truncate_prompt_tokens=-1,
            use_beam_search=True,
            user="user",
            vllm_xargs={"foo": "string"},
        )
        assert_matches_type(object, chat, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create_completion(self, async_client: AsyncLightcone) -> None:
        response = await async_client.chat.with_raw_response.create_completion(
            messages=[
                {
                    "content": "string",
                    "role": "developer",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = await response.parse()
        assert_matches_type(object, chat, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create_completion(self, async_client: AsyncLightcone) -> None:
        async with async_client.chat.with_streaming_response.create_completion(
            messages=[
                {
                    "content": "string",
                    "role": "developer",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = await response.parse()
            assert_matches_type(object, chat, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .computer_drag_params import ComputerDragParams as ComputerDragParams
from .computer_html_params import ComputerHTMLParams as ComputerHTMLParams
from .computer_list_params import ComputerListParams as ComputerListParams
from .computer_type_params import ComputerTypeParams as ComputerTypeParams
from .computer_batch_params import ComputerBatchParams as ComputerBatchParams
from .computer_click_params import ComputerClickParams as ComputerClickParams
from .computer_debug_params import ComputerDebugParams as ComputerDebugParams
from .computer_create_params import ComputerCreateParams as ComputerCreateParams
from .computer_drag_response import ComputerDragResponse as ComputerDragResponse
from .computer_hotkey_params import ComputerHotkeyParams as ComputerHotkeyParams
from .computer_html_response import ComputerHTMLResponse as ComputerHTMLResponse
from .computer_key_up_params import ComputerKeyUpParams as ComputerKeyUpParams
from .computer_list_response import ComputerListResponse as ComputerListResponse
from .computer_scroll_params import ComputerScrollParams as ComputerScrollParams
from .computer_type_response import ComputerTypeResponse as ComputerTypeResponse
from .response_create_params import ResponseCreateParams as ResponseCreateParams
from .computer_batch_response import ComputerBatchResponse as ComputerBatchResponse
from .computer_click_response import ComputerClickResponse as ComputerClickResponse
from .computer_debug_response import ComputerDebugResponse as ComputerDebugResponse
from .computer_execute_params import ComputerExecuteParams as ComputerExecuteParams
from .computer_create_response import ComputerCreateResponse as ComputerCreateResponse
from .computer_hotkey_response import ComputerHotkeyResponse as ComputerHotkeyResponse
from .computer_key_down_params import ComputerKeyDownParams as ComputerKeyDownParams
from .computer_key_up_response import ComputerKeyUpResponse as ComputerKeyUpResponse
from .computer_mouse_up_params import ComputerMouseUpParams as ComputerMouseUpParams
from .computer_navigate_params import ComputerNavigateParams as ComputerNavigateParams
from .computer_scroll_response import ComputerScrollResponse as ComputerScrollResponse
from .computer_viewport_params import ComputerViewportParams as ComputerViewportParams
from .computer_execute_response import ComputerExecuteResponse as ComputerExecuteResponse
from .computer_key_down_response import ComputerKeyDownResponse as ComputerKeyDownResponse
from .computer_mouse_down_params import ComputerMouseDownParams as ComputerMouseDownParams
from .computer_mouse_up_response import ComputerMouseUpResponse as ComputerMouseUpResponse
from .computer_navigate_response import ComputerNavigateResponse as ComputerNavigateResponse
from .computer_retrieve_response import ComputerRetrieveResponse as ComputerRetrieveResponse
from .computer_screenshot_params import ComputerScreenshotParams as ComputerScreenshotParams
from .computer_viewport_response import ComputerViewportResponse as ComputerViewportResponse
from .computer_keepalive_response import ComputerKeepaliveResponse as ComputerKeepaliveResponse
from .computer_right_click_params import ComputerRightClickParams as ComputerRightClickParams
from .computer_change_proxy_params import ComputerChangeProxyParams as ComputerChangeProxyParams
from .computer_double_click_params import ComputerDoubleClickParams as ComputerDoubleClickParams
from .computer_mouse_down_response import ComputerMouseDownResponse as ComputerMouseDownResponse
from .computer_screenshot_response import ComputerScreenshotResponse as ComputerScreenshotResponse
from .chat_create_completion_params import ChatCreateCompletionParams as ChatCreateCompletionParams
from .computer_right_click_response import ComputerRightClickResponse as ComputerRightClickResponse
from .computer_change_proxy_response import ComputerChangeProxyResponse as ComputerChangeProxyResponse
from .computer_double_click_response import ComputerDoubleClickResponse as ComputerDoubleClickResponse
from .computer_retrieve_status_response import ComputerRetrieveStatusResponse as ComputerRetrieveStatusResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .._types import SequenceNotStr

__all__ = [
    "ChatCreateCompletionParams",
    "Message",
    "MessageChatCompletionDeveloperMessageParam",
    "MessageChatCompletionDeveloperMessageParamContentUnionMember1",
    "MessageChatCompletionSystemMessageParam",
    "MessageChatCompletionSystemMessageParamContentUnionMember1",
    "MessageChatCompletionUserMessageParam",
    "MessageChatCompletionUserMessageParamContentUnionMember1",
    "MessageChatCompletionUserMessageParamContentUnionMember1ChatCompletionContentPartTextParam",
    "MessageChatCompletionUserMessageParamContentUnionMember1ChatCompletionContentPartImageParam",
    "MessageChatCompletionUserMessageParamContentUnionMember1ChatCompletionContentPartImageParamImageURL",
    "MessageChatCompletionUserMessageParamContentUnionMember1ChatCompletionContentPartInputAudioParam",
    "MessageChatCompletionUserMessageParamContentUnionMember1ChatCompletionContentPartInputAudioParamInputAudio",
    "MessageChatCompletionUserMessageParamContentUnionMember1File",
    "MessageChatCompletionUserMessageParamContentUnionMember1FileFile",
    "MessageChatCompletionAssistantMessageParam",
    "MessageChatCompletionAssistantMessageParamAudio",
    "MessageChatCompletionAssistantMessageParamContentUnionMember1",
    "MessageChatCompletionAssistantMessageParamContentUnionMember1ChatCompletionContentPartTextParam",
    "MessageChatCompletionAssistantMessageParamContentUnionMember1ChatCompletionContentPartRefusalParam",
    "MessageChatCompletionAssistantMessageParamFunctionCall",
    "MessageChatCompletionAssistantMessageParamToolCall",
    "MessageChatCompletionAssistantMessageParamToolCallChatCompletionMessageFunctionToolCallParam",
    "MessageChatCompletionAssistantMessageParamToolCallChatCompletionMessageFunctionToolCallParamFunction",
    "MessageChatCompletionAssistantMessageParamToolCallChatCompletionMessageCustomToolCallParam",
    "MessageChatCompletionAssistantMessageParamToolCallChatCompletionMessageCustomToolCallParamCustom",
    "MessageChatCompletionToolMessageParam",
    "MessageChatCompletionToolMessageParamContentUnionMember1",
    "MessageChatCompletionFunctionMessageParam",
    "MessageCustomChatCompletionMessageParam",
    "MessageCustomChatCompletionMessageParamContentUnionMember1",
    "MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartTextParam",
    "MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartImageParam",
    "MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartImageParamImageURL",
    "MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartInputAudioParam",
    "MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartInputAudioParamInputAudio",
    "MessageCustomChatCompletionMessageParamContentUnionMember1File",
    "MessageCustomChatCompletionMessageParamContentUnionMember1FileFile",
    "MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartAudioParam",
    "MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartAudioParamAudioURL",
    "MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartVideoParam",
    "MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartVideoParamVideoURL",
    "MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartRefusalParam",
    "MessageCustomChatCompletionMessageParamContentUnionMember1CustomChatCompletionContentSimpleImageParam",
    "MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartImageEmbedsParam",
    "MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartAudioEmbedsParam",
    "MessageCustomChatCompletionMessageParamContentUnionMember1CustomChatCompletionContentSimpleAudioParam",
    "MessageCustomChatCompletionMessageParamContentUnionMember1CustomChatCompletionContentSimpleVideoParam",
    "MessageCustomChatCompletionMessageParamContentUnionMember1CustomThinkCompletionContentParam",
    "MessageCustomChatCompletionMessageParamToolCall",
    "MessageCustomChatCompletionMessageParamToolCallFunction",
    "MessageCustomChatCompletionMessageParamTool",
    "MessageCustomChatCompletionMessageParamToolFunction",
    "MessageMessage",
    "MessageMessageAuthor",
    "LogitsProcessor",
    "LogitsProcessorLogitsProcessorConstructor",
    "ResponseFormat",
    "ResponseFormatResponseFormat",
    "ResponseFormatResponseFormatJsonSchema",
    "ResponseFormatStructuralTagResponseFormat",
    "ResponseFormatLegacyStructuralTagResponseFormat",
    "ResponseFormatLegacyStructuralTagResponseFormatStructure",
    "StreamOptions",
    "StructuredOutputs",
    "ToolChoice",
    "ToolChoiceChatCompletionNamedToolChoiceParam",
    "ToolChoiceChatCompletionNamedToolChoiceParamFunction",
    "Tool",
    "ToolFunction",
]


class ChatCreateCompletionParams(TypedDict, total=False):
    messages: Required[Iterable[Message]]

    add_generation_prompt: bool
    """If true, the generation prompt will be added to the chat template.

    This is a parameter used by chat template in tokenizer config of the model.
    """

    add_special_tokens: bool
    """If true, special tokens (e.g.

    BOS) will be added to the prompt on top of what is added by the chat template.
    For most models, the chat template takes care of adding the special tokens so
    this should be set to false (as is the default).
    """

    allowed_token_ids: Optional[Iterable[int]]

    bad_words: SequenceNotStr[str]

    cache_salt: Optional[str]
    """
    If specified, the prefix cache will be salted with the provided string to
    prevent an attacker to guess prompts in multi-user environments. The salt should
    be random, protected from access by 3rd parties, and long enough to be
    unpredictable (e.g., 43 characters base64-encoded, corresponding to 256 bit).
    """

    chat_template: Optional[str]
    """A Jinja template to use for this conversion.

    As of transformers v4.44, default chat template is no longer allowed, so you
    must provide a chat template if the tokenizer does not define one.
    """

    chat_template_kwargs: Optional[Dict[str, object]]
    """Additional keyword args to pass to the template renderer.

    Will be accessible by the chat template.
    """

    continue_final_message: bool
    """
    If this is set, the chat will be formatted so that the final message in the chat
    is open-ended, without any EOS tokens. The model will continue this message
    rather than starting a new one. This allows you to "prefill" part of the model's
    response for it. Cannot be used at the same time as `add_generation_prompt`.
    """

    documents: Optional[Iterable[Dict[str, str]]]
    """
    A list of dicts representing documents that will be accessible to the model if
    it is performing RAG (retrieval-augmented generation). If the template does not
    support RAG, this argument will have no effect. We recommend that each document
    should be a dict containing "title" and "text" keys.
    """

    echo: bool
    """
    If true, the new message will be prepended with the last message if they belong
    to the same role.
    """

    frequency_penalty: Optional[float]

    ignore_eos: bool

    include_reasoning: bool

    include_stop_str_in_output: bool

    kv_transfer_params: Optional[Dict[str, object]]
    """KVTransfer parameters used for disaggregated serving."""

    length_penalty: float

    logit_bias: Optional[Dict[str, float]]

    logits_processors: Optional[SequenceNotStr[LogitsProcessor]]
    """
    A list of either qualified names of logits processors, or constructor objects,
    to apply when sampling. A constructor is a JSON object with a required
    'qualname' field specifying the qualified name of the processor class/factory,
    and optional 'args' and 'kwargs' fields containing positional and keyword
    arguments. For example: {'qualname': 'my_module.MyLogitsProcessor', 'args': [1,
    2], 'kwargs': {'param': 'value'}}.
    """

    logprobs: Optional[bool]

    max_completion_tokens: Optional[int]

    max_tokens: Optional[int]

    min_p: Optional[float]

    min_tokens: int

    mm_processor_kwargs: Optional[Dict[str, object]]
    """Additional kwargs to pass to the HF processor."""

    model: Optional[str]

    n: Optional[int]

    parallel_tool_calls: Optional[bool]

    presence_penalty: Optional[float]

    priority: int
    """The priority of the request (lower means earlier handling; default: 0).

    Any priority other than 0 will raise an error if the served model does not use
    priority scheduling.
    """

    prompt_logprobs: Optional[int]

    reasoning_effort: Optional[Literal["low", "medium", "high"]]

    repetition_penalty: Optional[float]

    request_id: str
    """The request_id related to this request.

    If the caller does not set it, a random_uuid will be generated. This id is used
    through out the inference process and return in response.
    """

    response_format: Optional[ResponseFormat]

    return_token_ids: Optional[bool]
    """If specified, the result will include token IDs alongside the generated text.

    In streaming mode, prompt_token_ids is included only in the first chunk, and
    token_ids contains the delta tokens for each chunk. This is useful for debugging
    or when you need to map generated text back to input tokens.
    """

    return_tokens_as_token_ids: Optional[bool]
    """
    If specified with 'logprobs', tokens are represented as strings of the form
    'token_id:{token_id}' so that tokens that are not JSON-encodable can be
    identified.
    """

    seed: Optional[int]

    skip_special_tokens: bool

    spaces_between_special_tokens: bool

    stop: Union[str, SequenceNotStr[str], None]

    stop_token_ids: Optional[Iterable[int]]

    stream: Optional[bool]

    stream_options: Optional[StreamOptions]

    structured_outputs: Optional[StructuredOutputs]
    """Additional kwargs for structured outputs"""

    temperature: Optional[float]

    tool_choice: Optional[ToolChoice]

    tools: Optional[Iterable[Tool]]

    top_k: Optional[int]

    top_logprobs: Optional[int]

    top_p: Optional[float]

    truncate_prompt_tokens: Optional[int]

    use_beam_search: bool

    user: Optional[str]

    vllm_xargs: Optional[Dict[str, Union[str, float, SequenceNotStr[Union[str, float]]]]]
    """
    Additional request parameters with (list of) string or numeric values, used by
    custom extensions.
    """


class MessageChatCompletionDeveloperMessageParamContentUnionMember1(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    Learn about [text inputs](https://platform.openai.com/docs/guides/text-generation).
    """

    text: Required[str]

    type: Required[Literal["text"]]


class MessageChatCompletionDeveloperMessageParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    Developer-provided instructions that the model should follow, regardless of
    messages sent by the user. With o1 models and newer, `developer` messages
    replace the previous `system` messages.
    """

    content: Required[Union[str, Iterable[MessageChatCompletionDeveloperMessageParamContentUnionMember1]]]

    role: Required[Literal["developer"]]

    name: str


class MessageChatCompletionSystemMessageParamContentUnionMember1(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    Learn about [text inputs](https://platform.openai.com/docs/guides/text-generation).
    """

    text: Required[str]

    type: Required[Literal["text"]]


class MessageChatCompletionSystemMessageParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    Developer-provided instructions that the model should follow, regardless of
    messages sent by the user. With o1 models and newer, use `developer` messages
    for this purpose instead.
    """

    content: Required[Union[str, Iterable[MessageChatCompletionSystemMessageParamContentUnionMember1]]]

    role: Required[Literal["system"]]

    name: str


class MessageChatCompletionUserMessageParamContentUnionMember1ChatCompletionContentPartTextParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    """
    Learn about [text inputs](https://platform.openai.com/docs/guides/text-generation).
    """

    text: Required[str]

    type: Required[Literal["text"]]


class MessageChatCompletionUserMessageParamContentUnionMember1ChatCompletionContentPartImageParamImageURL(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    url: Required[str]

    detail: Literal["auto", "low", "high"]


class MessageChatCompletionUserMessageParamContentUnionMember1ChatCompletionContentPartImageParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    """Learn about [image inputs](https://platform.openai.com/docs/guides/vision)."""

    image_url: Required[
        MessageChatCompletionUserMessageParamContentUnionMember1ChatCompletionContentPartImageParamImageURL
    ]

    type: Required[Literal["image_url"]]


class MessageChatCompletionUserMessageParamContentUnionMember1ChatCompletionContentPartInputAudioParamInputAudio(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    data: Required[str]

    format: Required[Literal["wav", "mp3"]]


class MessageChatCompletionUserMessageParamContentUnionMember1ChatCompletionContentPartInputAudioParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    """Learn about [audio inputs](https://platform.openai.com/docs/guides/audio)."""

    input_audio: Required[
        MessageChatCompletionUserMessageParamContentUnionMember1ChatCompletionContentPartInputAudioParamInputAudio
    ]

    type: Required[Literal["input_audio"]]


class MessageChatCompletionUserMessageParamContentUnionMember1FileFile(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    file_data: str

    file_id: str

    filename: str


class MessageChatCompletionUserMessageParamContentUnionMember1File(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    Learn about [file inputs](https://platform.openai.com/docs/guides/text) for text generation.
    """

    file: Required[MessageChatCompletionUserMessageParamContentUnionMember1FileFile]

    type: Required[Literal["file"]]


MessageChatCompletionUserMessageParamContentUnionMember1: TypeAlias = Union[
    MessageChatCompletionUserMessageParamContentUnionMember1ChatCompletionContentPartTextParam,
    MessageChatCompletionUserMessageParamContentUnionMember1ChatCompletionContentPartImageParam,
    MessageChatCompletionUserMessageParamContentUnionMember1ChatCompletionContentPartInputAudioParam,
    MessageChatCompletionUserMessageParamContentUnionMember1File,
]


class MessageChatCompletionUserMessageParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    Messages sent by an end user, containing prompts or additional context
    information.
    """

    content: Required[Union[str, Iterable[MessageChatCompletionUserMessageParamContentUnionMember1]]]

    role: Required[Literal["user"]]

    name: str


class MessageChatCompletionAssistantMessageParamAudio(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    Data about a previous audio response from the model.
    [Learn more](https://platform.openai.com/docs/guides/audio).
    """

    id: Required[str]


class MessageChatCompletionAssistantMessageParamContentUnionMember1ChatCompletionContentPartTextParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    """
    Learn about [text inputs](https://platform.openai.com/docs/guides/text-generation).
    """

    text: Required[str]

    type: Required[Literal["text"]]


class MessageChatCompletionAssistantMessageParamContentUnionMember1ChatCompletionContentPartRefusalParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    refusal: Required[str]

    type: Required[Literal["refusal"]]


MessageChatCompletionAssistantMessageParamContentUnionMember1: TypeAlias = Union[
    MessageChatCompletionAssistantMessageParamContentUnionMember1ChatCompletionContentPartTextParam,
    MessageChatCompletionAssistantMessageParamContentUnionMember1ChatCompletionContentPartRefusalParam,
]


class MessageChatCompletionAssistantMessageParamFunctionCall(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Deprecated and replaced by `tool_calls`.

    The name and arguments of a function that should be called, as generated by the model.
    """

    arguments: Required[str]

    name: Required[str]


class MessageChatCompletionAssistantMessageParamToolCallChatCompletionMessageFunctionToolCallParamFunction(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    """The function that the model called."""

    arguments: Required[str]

    name: Required[str]


class MessageChatCompletionAssistantMessageParamToolCallChatCompletionMessageFunctionToolCallParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    """A call to a function tool created by the model."""

    id: Required[str]

    function: Required[
        MessageChatCompletionAssistantMessageParamToolCallChatCompletionMessageFunctionToolCallParamFunction
    ]
    """The function that the model called."""

    type: Required[Literal["function"]]


class MessageChatCompletionAssistantMessageParamToolCallChatCompletionMessageCustomToolCallParamCustom(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    """The custom tool that the model called."""

    input: Required[str]

    name: Required[str]


class MessageChatCompletionAssistantMessageParamToolCallChatCompletionMessageCustomToolCallParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    """A call to a custom tool created by the model."""

    id: Required[str]

    custom: Required[MessageChatCompletionAssistantMessageParamToolCallChatCompletionMessageCustomToolCallParamCustom]
    """The custom tool that the model called."""

    type: Required[Literal["custom"]]


MessageChatCompletionAssistantMessageParamToolCall: TypeAlias = Union[
    MessageChatCompletionAssistantMessageParamToolCallChatCompletionMessageFunctionToolCallParam,
    MessageChatCompletionAssistantMessageParamToolCallChatCompletionMessageCustomToolCallParam,
]


class MessageChatCompletionAssistantMessageParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Messages sent by the model in response to user messages."""

    role: Required[Literal["assistant"]]

    audio: Optional[MessageChatCompletionAssistantMessageParamAudio]
    """
    Data about a previous audio response from the model.
    [Learn more](https://platform.openai.com/docs/guides/audio).
    """

    content: Union[str, Iterable[MessageChatCompletionAssistantMessageParamContentUnionMember1], None]

    function_call: Optional[MessageChatCompletionAssistantMessageParamFunctionCall]
    """Deprecated and replaced by `tool_calls`.

    The name and arguments of a function that should be called, as generated by the
    model.
    """

    name: str

    refusal: Optional[str]

    tool_calls: Iterable[MessageChatCompletionAssistantMessageParamToolCall]


class MessageChatCompletionToolMessageParamContentUnionMember1(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    Learn about [text inputs](https://platform.openai.com/docs/guides/text-generation).
    """

    text: Required[str]

    type: Required[Literal["text"]]


class MessageChatCompletionToolMessageParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    content: Required[Union[str, Iterable[MessageChatCompletionToolMessageParamContentUnionMember1]]]

    role: Required[Literal["tool"]]

    tool_call_id: Required[str]


class MessageChatCompletionFunctionMessageParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    content: Required[Optional[str]]

    name: Required[str]

    role: Required[Literal["function"]]


class MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartTextParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    """
    Learn about [text inputs](https://platform.openai.com/docs/guides/text-generation).
    """

    text: Required[str]

    type: Required[Literal["text"]]


class MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartImageParamImageURL(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    url: Required[str]

    detail: Literal["auto", "low", "high"]


class MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartImageParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    """Learn about [image inputs](https://platform.openai.com/docs/guides/vision)."""

    image_url: Required[
        MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartImageParamImageURL
    ]

    type: Required[Literal["image_url"]]


class MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartInputAudioParamInputAudio(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    data: Required[str]

    format: Required[Literal["wav", "mp3"]]


class MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartInputAudioParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    """Learn about [audio inputs](https://platform.openai.com/docs/guides/audio)."""

    input_audio: Required[
        MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartInputAudioParamInputAudio
    ]

    type: Required[Literal["input_audio"]]


class MessageCustomChatCompletionMessageParamContentUnionMember1FileFile(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    file_data: str

    file_id: str

    filename: str


class MessageCustomChatCompletionMessageParamContentUnionMember1File(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    Learn about [file inputs](https://platform.openai.com/docs/guides/text) for text generation.
    """

    file: Required[MessageCustomChatCompletionMessageParamContentUnionMember1FileFile]

    type: Required[Literal["file"]]


class MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartAudioParamAudioURL(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    url: Required[str]


class MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartAudioParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    audio_url: Required[
        MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartAudioParamAudioURL
    ]

    type: Required[Literal["audio_url"]]


class MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartVideoParamVideoURL(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    url: Required[str]


class MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartVideoParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    type: Required[Literal["video_url"]]

    video_url: Required[
        MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartVideoParamVideoURL
    ]


class MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartRefusalParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    refusal: Required[str]

    type: Required[Literal["refusal"]]


class MessageCustomChatCompletionMessageParamContentUnionMember1CustomChatCompletionContentSimpleImageParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    """
    A simpler version of the param that only accepts a plain image_url.
    This is supported by OpenAI API, although it is not documented.

    Example:
    {
        "image_url": "https://example.com/image.jpg"
    }
    """

    image_url: Optional[str]

    uuid: Optional[str]


class MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartImageEmbedsParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    type: Required[Literal["image_embeds"]]

    image_embeds: Union[str, Dict[str, str], None]

    uuid: Optional[str]


class MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartAudioEmbedsParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    type: Required[Literal["audio_embeds"]]

    audio_embeds: Union[str, Dict[str, str], None]

    uuid: Optional[str]


class MessageCustomChatCompletionMessageParamContentUnionMember1CustomChatCompletionContentSimpleAudioParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    """A simpler version of the param that only accepts a plain audio_url.

    Example:
    {
        "audio_url": "https://example.com/audio.mp3"
    }
    """

    audio_url: Optional[str]


class MessageCustomChatCompletionMessageParamContentUnionMember1CustomChatCompletionContentSimpleVideoParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    """A simpler version of the param that only accepts a plain audio_url.

    Example:
    {
        "video_url": "https://example.com/video.mp4"
    }
    """

    uuid: Optional[str]

    video_url: Optional[str]


class MessageCustomChatCompletionMessageParamContentUnionMember1CustomThinkCompletionContentParam(
    TypedDict, total=False, extra_items=object
):  # type: ignore[call-arg]
    """A Think Completion Content Param that accepts a plain text and a boolean.

    Example:
    {
        "thinking": "I am thinking about the answer",
        "closed": True,
        "type": "thinking"
    }
    """

    thinking: Required[str]

    type: Required[Literal["thinking"]]

    closed: bool


MessageCustomChatCompletionMessageParamContentUnionMember1: TypeAlias = Union[
    MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartTextParam,
    MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartImageParam,
    MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartInputAudioParam,
    MessageCustomChatCompletionMessageParamContentUnionMember1File,
    MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartAudioParam,
    MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartVideoParam,
    MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartRefusalParam,
    MessageCustomChatCompletionMessageParamContentUnionMember1CustomChatCompletionContentSimpleImageParam,
    MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartImageEmbedsParam,
    MessageCustomChatCompletionMessageParamContentUnionMember1ChatCompletionContentPartAudioEmbedsParam,
    MessageCustomChatCompletionMessageParamContentUnionMember1CustomChatCompletionContentSimpleAudioParam,
    MessageCustomChatCompletionMessageParamContentUnionMember1CustomChatCompletionContentSimpleVideoParam,
    str,
    MessageCustomChatCompletionMessageParamContentUnionMember1CustomThinkCompletionContentParam,
]


class MessageCustomChatCompletionMessageParamToolCallFunction(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The function that the model called."""

    arguments: Required[str]

    name: Required[str]


class MessageCustomChatCompletionMessageParamToolCall(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A call to a function tool created by the model."""

    id: Required[str]

    function: Required[MessageCustomChatCompletionMessageParamToolCallFunction]
    """The function that the model called."""

    type: Required[Literal["function"]]


class MessageCustomChatCompletionMessageParamToolFunction(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    name: Required[str]

    description: str

    parameters: Dict[str, object]

    strict: Optional[bool]


class MessageCustomChatCompletionMessageParamTool(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A function tool that can be used to generate a response."""

    function: Required[MessageCustomChatCompletionMessageParamToolFunction]

    type: Required[Literal["function"]]


class MessageCustomChatCompletionMessageParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Enables custom roles in the Chat Completion API."""

    role: Required[str]

    content: Union[str, SequenceNotStr[MessageCustomChatCompletionMessageParamContentUnionMember1]]

    name: str

    reasoning: Optional[str]

    tool_call_id: Optional[str]

    tool_calls: Optional[Iterable[MessageCustomChatCompletionMessageParamToolCall]]

    tools: Optional[Iterable[MessageCustomChatCompletionMessageParamTool]]


class MessageMessageAuthor(TypedDict, total=False):
    role: Required[Literal["user", "assistant", "system", "developer", "tool"]]
    """The role of a message author (mirrors `chat::Role`)."""

    name: Optional[str]


class MessageMessage(TypedDict, total=False):
    author: Required[MessageMessageAuthor]

    channel: Optional[str]

    content: Iterable[object]

    content_type: Optional[str]

    recipient: Optional[str]


Message: TypeAlias = Union[
    MessageChatCompletionDeveloperMessageParam,
    MessageChatCompletionSystemMessageParam,
    MessageChatCompletionUserMessageParam,
    MessageChatCompletionAssistantMessageParam,
    MessageChatCompletionToolMessageParam,
    MessageChatCompletionFunctionMessageParam,
    MessageCustomChatCompletionMessageParam,
    MessageMessage,
]


class LogitsProcessorLogitsProcessorConstructor(TypedDict, total=False):
    qualname: Required[str]

    args: Optional[Iterable[object]]

    kwargs: Optional[Dict[str, object]]


LogitsProcessor: TypeAlias = Union[str, LogitsProcessorLogitsProcessorConstructor]


class ResponseFormatResponseFormatJsonSchema(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    name: Required[str]

    description: Optional[str]

    schema: Optional[Dict[str, object]]

    strict: Optional[bool]


class ResponseFormatResponseFormat(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    type: Required[Literal["text", "json_object", "json_schema"]]

    json_schema: Optional[ResponseFormatResponseFormatJsonSchema]


class ResponseFormatStructuralTagResponseFormat(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    format: Required[object]

    type: Required[Literal["structural_tag"]]


class ResponseFormatLegacyStructuralTagResponseFormatStructure(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    begin: Required[str]

    end: Required[str]

    schema: Optional[Dict[str, object]]


class ResponseFormatLegacyStructuralTagResponseFormat(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    structures: Required[Iterable[ResponseFormatLegacyStructuralTagResponseFormatStructure]]

    triggers: Required[SequenceNotStr[str]]

    type: Required[Literal["structural_tag"]]


ResponseFormat: TypeAlias = Union[
    ResponseFormatResponseFormat,
    ResponseFormatStructuralTagResponseFormat,
    ResponseFormatLegacyStructuralTagResponseFormat,
]


class StreamOptions(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    continuous_usage_stats: Optional[bool]

    include_usage: Optional[bool]


class StructuredOutputs(TypedDict, total=False):
    """Additional kwargs for structured outputs"""

    _backend: Optional[str]

    _backend_was_auto: bool

    choice: Optional[SequenceNotStr[str]]

    disable_additional_properties: bool

    disable_any_whitespace: bool

    disable_fallback: bool

    grammar: Optional[str]

    json: Union[str, Dict[str, object], None]

    json_object: Optional[bool]

    regex: Optional[str]

    structural_tag: Optional[str]

    whitespace_pattern: Optional[str]


class ToolChoiceChatCompletionNamedToolChoiceParamFunction(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    name: Required[str]


class ToolChoiceChatCompletionNamedToolChoiceParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    function: Required[ToolChoiceChatCompletionNamedToolChoiceParamFunction]

    type: Literal["function"]


ToolChoice: TypeAlias = Union[Literal["none", "auto", "required"], ToolChoiceChatCompletionNamedToolChoiceParam]


class ToolFunction(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    name: Required[str]

    description: Optional[str]

    parameters: Optional[Dict[str, object]]


class Tool(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    function: Required[ToolFunction]

    type: Literal["function"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tzafon import Lightcone, AsyncLightcone
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestResponses:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Lightcone) -> None:
        response = client.responses.create(
            input="string",
        )
        assert_matches_type(object, response, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Lightcone) -> None:
        response = client.responses.create(
            input="string",
            background=True,
            cache_salt="cache_salt",
            enable_response_messages=True,
            include=["code_interpreter_call.outputs"],
            include_stop_str_in_output=True,
            instructions="instructions",
            logit_bias={"foo": 0},
            max_output_tokens=0,
            max_tool_calls=0,
            metadata={"foo": "string"},
            mm_processor_kwargs={"foo": "bar"},
            model="model",
            parallel_tool_calls=True,
            previous_input_messages=[
                {
                    "author": {
                        "role": "user",
                        "name": "name",
                    },
                    "channel": "channel",
                    "content": [{}],
                    "content_type": "content_type",
                    "recipient": "recipient",
                }
            ],
            previous_response_id="previous_response_id",
            priority=0,
            prompt={
                "id": "id",
                "variables": {"foo": "string"},
                "version": "version",
            },
            prompt_cache_key="prompt_cache_key",
            reasoning={
                "effort": "none",
                "generate_summary": "auto",
                "summary": "auto",
            },
            request_id="request_id",
            service_tier="auto",
            skip_special_tokens=True,
            store=True,
            stream=True,
            temperature=0,
            text={
                "format": {"type": "text"},
                "verbosity": "low",
            },
            tool_choice="none",
            tools=[
                {
                    "name": "name",
                    "type": "function",
                    "description": "description",
                    "parameters": {"foo": "bar"},
                    "strict": True,
                }
            ],
            top_k=0,
            top_logprobs=0,
            top_p=0,
            truncation="auto",
            user="user",
        )
        assert_matches_type(object, response, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Lightcone) -> None:
        http_response = client.responses.with_raw_response.create(
            input="string",
        )

        assert http_response.is_closed is True
        assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"
        response = http_response.parse()
        assert_matches_type(object, response, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Lightcone) -> None:
        with client.responses.with_streaming_response.create(
            input="string",
        ) as http_response:
            assert not http_response.is_closed
            assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"

            response = http_response.parse()
            assert_matches_type(object, response, path=["response"])

        assert cast(Any, http_response.is_closed) is True


class TestAsyncResponses:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncLightcone) -> None:
        response = await async_client.responses.create(
            input="string",
        )
        assert_matches_type(object, response, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncLightcone) -> None:
        response = await async_client.responses.create(
            input="string",
            background=True,
            cache_salt="cache_salt",
            enable_response_messages=True,
            include=["code_interpreter_call.outputs"],
            include_stop_str_in_output=True,
            instructions="instructions",
            logit_bias={"foo": 0},
            max_output_tokens=0,
            max_tool_calls=0,
            metadata={"foo": "string"},
            mm_processor_kwargs={"foo": "bar"},
            model="model",
            parallel_tool_calls=True,
            previous_input_messages=[
                {
                    "author": {
                        "role": "user",
                        "name": "name",
                    },
                    "channel": "channel",
                    "content": [{}],
                    "content_type": "content_type",
                    "recipient": "recipient",
                }
            ],
            previous_response_id="previous_response_id",
            priority=0,
            prompt={
                "id": "id",
                "variables": {"foo": "string"},
                "version": "version",
            },
            prompt_cache_key="prompt_cache_key",
            reasoning={
                "effort": "none",
                "generate_summary": "auto",
                "summary": "auto",
            },
            request_id="request_id",
            service_tier="auto",
            skip_special_tokens=True,
            store=True,
            stream=True,
            temperature=0,
            text={
                "format": {"type": "text"},
                "verbosity": "low",
            },
            tool_choice="none",
            tools=[
                {
                    "name": "name",
                    "type": "function",
                    "description": "description",
                    "parameters": {"foo": "bar"},
                    "strict": True,
                }
            ],
            top_k=0,
            top_logprobs=0,
            top_p=0,
            truncation="auto",
            user="user",
        )
        assert_matches_type(object, response, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncLightcone) -> None:
        http_response = await async_client.responses.with_raw_response.create(
            input="string",
        )

        assert http_response.is_closed is True
        assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"
        response = await http_response.parse()
        assert_matches_type(object, response, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncLightcone) -> None:
        async with async_client.responses.with_streaming_response.create(
            input="string",
        ) as http_response:
            assert not http_response.is_closed
            assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"

            response = await http_response.parse()
            assert_matches_type(object, response, path=["response"])

        assert cast(Any, http_response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["TaskRetrieveStatusResponse"]


class TaskRetrieveStatusResponse(BaseModel):
    exit_code: Optional[int] = None

    status: Optional[str] = None

    task_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._models import SecurityOptions
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError, LightconeError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import chat, agent, models, computers, responses
    from .resources.chat import ChatResource, AsyncChatResource
    from .resources.models import ModelsResource, AsyncModelsResource
    from .resources.responses import ResponsesResource, AsyncResponsesResource
    from .lib.computer_session import ComputerSession, ComputerResource
    from .resources.agent.agent import AgentResource, AsyncAgentResource
    from .resources.computers.computers import ComputersResource, AsyncComputersResource

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Lightcone",
    "AsyncLightcone",
    "Client",
    "AsyncClient",
]


class Lightcone(SyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Lightcone client instance.

        This automatically infers the `api_key` argument from the `TZAFON_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("TZAFON_API_KEY")
        if api_key is None:
            raise LightconeError(
                "The api_key client option must be set either by passing api_key to the client or by setting the TZAFON_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("LIGHTCONE_BASE_URL")
        if base_url is None:
            base_url = f"https://api.tzafon.ai"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def agent(self) -> AgentResource:
        from .resources.agent import AgentResource

        return AgentResource(self)

    @cached_property
    def computers(self) -> ComputersResource:
        from .resources.computers import ComputersResource

        return ComputersResource(self)

    @cached_property
    def computer(self) -> ComputerResource:
        from .lib.computer_session import ComputerResource

        return ComputerResource(self)

    def create(self, kind: str = "browser", **kwargs: Any) -> ComputerSession:
        """Create a computer session (backwards compat with computer-python SDK)."""
        return self.computer.create(kind=kind, **kwargs)

    @cached_property
    def responses(self) -> ResponsesResource:
        from .resources.responses import ResponsesResource

        return ResponsesResource(self)

    @cached_property
    def chat(self) -> ChatResource:
        from .resources.chat import ChatResource

        return ChatResource(self)

    @cached_property
    def models(self) -> ModelsResource:
        from .resources.models import ModelsResource

        return ModelsResource(self)

    @cached_property
    def with_raw_response(self) -> LightconeWithRawResponse:
        return LightconeWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LightconeWithStreamedResponse:
        return LightconeWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @override
    def _auth_headers(self, security: SecurityOptions) -> dict[str, str]:
        return {
            **(self._api_key_auth if security.get("api_key_auth", False) else {}),
        }

    @property
    def _api_key_auth(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncLightcone(AsyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncLightcone client instance.

        This automatically infers the `api_key` argument from the `TZAFON_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("TZAFON_API_KEY")
        if api_key is None:
            raise LightconeError(
                "The api_key client option must be set either by passing api_key to the client or by setting the TZAFON_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("LIGHTCONE_BASE_URL")
        if base_url is None:
            base_url = f"https://api.tzafon.ai"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def agent(self) -> AsyncAgentResource:
        from .resources.agent import AsyncAgentResource

        return AsyncAgentResource(self)

    @cached_property
    def computers(self) -> AsyncComputersResource:
        from .resources.computers import AsyncComputersResource

        return AsyncComputersResource(self)

    @cached_property
    def responses(self) -> AsyncResponsesResource:
        from .resources.responses import AsyncResponsesResource

        return AsyncResponsesResource(self)

    @cached_property
    def chat(self) -> AsyncChatResource:
        from .resources.chat import AsyncChatResource

        return AsyncChatResource(self)

    @cached_property
    def models(self) -> AsyncModelsResource:
        from .resources.models import AsyncModelsResource

        return AsyncModelsResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncLightconeWithRawResponse:
        return AsyncLightconeWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLightconeWithStreamedResponse:
        return AsyncLightconeWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @override
    def _auth_headers(self, security: SecurityOptions) -> dict[str, str]:
        return {
            **(self._api_key_auth if security.get("api_key_auth", False) else {}),
        }

    @property
    def _api_key_auth(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class LightconeWithRawResponse:
    _client: Lightcone

    def __init__(self, client: Lightcone) -> None:
        self._client = client

    @cached_property
    def agent(self) -> agent.AgentResourceWithRawResponse:
        from .resources.agent import AgentResourceWithRawResponse

        return AgentResourceWithRawResponse(self._client.agent)

    @cached_property
    def computers(self) -> computers.ComputersResourceWithRawResponse:
        from .resources.computers import ComputersResourceWithRawResponse

        return ComputersResourceWithRawResponse(self._client.computers)

    @cached_property
    def responses(self) -> responses.ResponsesResourceWithRawResponse:
        from .resources.responses import ResponsesResourceWithRawResponse

        return ResponsesResourceWithRawResponse(self._client.responses)

    @cached_property
    def chat(self) -> chat.ChatResourceWithRawResponse:
        from .resources.chat import ChatResourceWithRawResponse

        return ChatResourceWithRawResponse(self._client.chat)

    @cached_property
    def models(self) -> models.ModelsResourceWithRawResponse:
        from .resources.models import ModelsResourceWithRawResponse

        return ModelsResourceWithRawResponse(self._client.models)


class AsyncLightconeWithRawResponse:
    _client: AsyncLightcone

    def __init__(self, client: AsyncLightcone) -> None:
        self._client = client

    @cached_property
    def agent(self) -> agent.AsyncAgentResourceWithRawResponse:
        from .resources.agent import AsyncAgentResourceWithRawResponse

        return AsyncAgentResourceWithRawResponse(self._client.agent)

    @cached_property
    def computers(self) -> computers.AsyncComputersResourceWithRawResponse:
        from .resources.computers import AsyncComputersResourceWithRawResponse

        return AsyncComputersResourceWithRawResponse(self._client.computers)

    @cached_property
    def responses(self) -> responses.AsyncResponsesResourceWithRawResponse:
        from .resources.responses import AsyncResponsesResourceWithRawResponse

        return AsyncResponsesResourceWithRawResponse(self._client.responses)

    @cached_property
    def chat(self) -> chat.AsyncChatResourceWithRawResponse:
        from .resources.chat import AsyncChatResourceWithRawResponse

        return AsyncChatResourceWithRawResponse(self._client.chat)

    @cached_property
    def models(self) -> models.AsyncModelsResourceWithRawResponse:
        from .resources.models import AsyncModelsResourceWithRawResponse

        return AsyncModelsResourceWithRawResponse(self._client.models)


class LightconeWithStreamedResponse:
    _client: Lightcone

    def __init__(self, client: Lightcone) -> None:
        self._client = client

    @cached_property
    def agent(self) -> agent.AgentResourceWithStreamingResponse:
        from .resources.agent import AgentResourceWithStreamingResponse

        return AgentResourceWithStreamingResponse(self._client.agent)

    @cached_property
    def computers(self) -> computers.ComputersResourceWithStreamingResponse:
        from .resources.computers import ComputersResourceWithStreamingResponse

        return ComputersResourceWithStreamingResponse(self._client.computers)

    @cached_property
    def responses(self) -> responses.ResponsesResourceWithStreamingResponse:
        from .resources.responses import ResponsesResourceWithStreamingResponse

        return ResponsesResourceWithStreamingResponse(self._client.responses)

    @cached_property
    def chat(self) -> chat.ChatResourceWithStreamingResponse:
        from .resources.chat import ChatResourceWithStreamingResponse

        return ChatResourceWithStreamingResponse(self._client.chat)

    @cached_property
    def models(self) -> models.ModelsResourceWithStreamingResponse:
        from .resources.models import ModelsResourceWithStreamingResponse

        return ModelsResourceWithStreamingResponse(self._client.models)


class AsyncLightconeWithStreamedResponse:
    _client: AsyncLightcone

    def __init__(self, client: AsyncLightcone) -> None:
        self._client = client

    @cached_property
    def agent(self) -> agent.AsyncAgentResourceWithStreamingResponse:
        from .resources.agent import AsyncAgentResourceWithStreamingResponse

        return AsyncAgentResourceWithStreamingResponse(self._client.agent)

    @cached_property
    def computers(self) -> computers.AsyncComputersResourceWithStreamingResponse:
        from .resources.computers import AsyncComputersResourceWithStreamingResponse

        return AsyncComputersResourceWithStreamingResponse(self._client.computers)

    @cached_property
    def responses(self) -> responses.AsyncResponsesResourceWithStreamingResponse:
        from .resources.responses import AsyncResponsesResourceWithStreamingResponse

        return AsyncResponsesResourceWithStreamingResponse(self._client.responses)

    @cached_property
    def chat(self) -> chat.AsyncChatResourceWithStreamingResponse:
        from .resources.chat import AsyncChatResourceWithStreamingResponse

        return AsyncChatResourceWithStreamingResponse(self._client.chat)

    @cached_property
    def models(self) -> models.AsyncModelsResourceWithStreamingResponse:
        from .resources.models import AsyncModelsResourceWithStreamingResponse

        return AsyncModelsResourceWithStreamingResponse(self._client.models)


Client = Lightcone

AsyncClient = AsyncLightcone
